/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

(function ()
{
  Ext.BLANK_IMAGE_URL = 'images/s.gif';

  Ext.Loader.setConfig( {
    enabled : true, // enables dynamic (on-demand) class-loading
    disableCaching : true, // false = removes the _dc param from all loader's HTTP requests
    paths : {
      'Ext' : 'js/extjs',
      'QDesk' : 'js/qdesk'
    }
  } );

  Ext.require( [
    'QDesk.Config',
    'QDesk.Log',
    'QDesk.Msg',
    'QDesk.Properties',

    'QDesk.enums.ExecNotificationRuleConditionType',
    'QDesk.enums.ExecStatus',
    'QDesk.enums.ExpressionLanguage',
    'QDesk.enums.JmxProtocol',
    'QDesk.enums.NotificationSeverity',
    'QDesk.enums.message.MessageChannel',
    'QDesk.enums.ProxyType',
    'QDesk.enums.SnmpV3AuthenticationProtocol',
    'QDesk.enums.SnmpV3PrivacyProtocol',
    'QDesk.enums.SnmpV3SecurityLevel',
    'QDesk.enums.WSAuthenticationType',
    'QDesk.enums.WSVersion',

    'QDesk.util.constant.ActionUrl',

    /* Scheduler actions */
    'QDesk.controller.command.scheduler.StartScheduler',
    'QDesk.controller.command.scheduler.PauseScheduler',
    'QDesk.controller.command.scheduler.StopScheduler',
    'QDesk.controller.command.connection.EditConnectionFolder',
    'QDesk.controller.command.connection.EditSchedulerConnection',
    'QDesk.controller.command.connection.DeleteConnectionFolder',
    'QDesk.controller.command.connection.DeleteSchedulerConnection',

    'QDesk.controller.command.scheduler.quartz.OpenSchedulerView',
    'QDesk.controller.command.settings.EditAppSettings',

    'QDesk.controller.command.scheduler.quartz.RefreshActiveTab',

    /* Jobs tab */
    'QDesk.controller.command.scheduler.quartz.jobs.InitializeJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobsTab',

    /* Currently Executing Jobs tab */
    'QDesk.controller.command.scheduler.quartz.jobs.InitializeExecutingJobsTab',
    'QDesk.controller.command.scheduler.quartz.jobs.RefreshExecutingJobsTab',

    /* Triggers tab */
    'QDesk.controller.command.scheduler.quartz.triggers.InitializeTriggersTab',
    'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggersTab',

    /* Info tab */
    'QDesk.controller.command.scheduler.quartz.info.InitializeInfoTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoTab',
    'QDesk.controller.command.scheduler.quartz.info.RefreshInfoForm',

    /* Exec history tab */
    'QDesk.controller.command.scheduler.quartz.history.InitializeSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryTab',
    'QDesk.controller.command.scheduler.quartz.history.RefreshSchedulerExecHistoryGrid',

    /* Statistics history tab */
    'QDesk.controller.command.scheduler.quartz.stats.RefreshSchedulerStatisticsTab',

    /* Exec notifications tab */
    'QDesk.controller.command.scheduler.quartz.notif.exec.InitializeSchedulerExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationsTab',
    'QDesk.controller.command.scheduler.quartz.notif.exec.RefreshSchedulerExecNotificationRulesGrid',

    'QDesk.store.quartz.JobTriggerGroupStore',

    'QDesk.store.quartz.stats.JobStatisticsTimePeriodStore',
    'QDesk.store.quartz.stats.JobStatisticsTypeStore',

    'QDesk.store.quartz.stats.SchedulerStatisticsTimePeriodStore',
    'QDesk.store.quartz.stats.SchedulerStatisticsTypeStore',

    'QDesk.store.quartz.stats.TriggerStatisticsTimePeriodStore',
    'QDesk.store.quartz.stats.TriggerStatisticsTypeStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.UxUtils',

    'QDesk.util.constant.ActionUrl',
    'QDesk.util.constant.DateFormat',

    'QDesk.view.Viewport',

    'Ext.data.Connection',
    'Ext.data.proxy.Ajax',
    'Ext.data.proxy.Server',

    'Ext.form.Basic'
  ] );


  /*
   * Register a global error handler for errors raised throughout the ExtJS framework and this application.
   * Note: to raise an error the application should use the Ext.Error.raise function.
   */
  Ext.Error.handle = function ( err )
  {
    Ext.log( {level : 'error', stack : true}, err );
    return QDesk.util.UxUtils.handleError( err );
  };


  Ext.application( {
    name : 'QDesk',
    appFolder : 'js/qdesk',

    controllers : [
      'navigation.HelpController',
      'navigation.NavigationController',

      'navigation.ConnectionsTreeController',
      'navigation.ConnectionsTreeToolbarController',

      'navigation.SettingsController',

      'scheduler.quartz.SchedulerController',

      'top.SwitchLocaleController',
      'top.LicenseInfoController'
    ],

    models : [
      'AboutData',
      'ConnectionsTreeNode',
      'JSScriptingObjectTreeNode',
      'Locale',
      'MessageMacroTreeNode',
      'NameValuePair',
      'RemoteJmxService',
      'Scheduler',
      'SettingsTreeNode',
      'TimeZone',

      'message.Message',
      'message.MessageAttachment',
      'message.MessageChannelProfile',
      'message.MessageChannelProfilesTreeNode',

      'quartz.Calendar',
      'quartz.ExecHistory',
      'quartz.ExecutingJob',
      'quartz.JobDataMapEntry',
      'quartz.JobDetail',
      'quartz.JobGroup',
      'quartz.Trigger',
      'quartz.TriggerGroup',
      'quartz.TriggerWithJobDetail',
      'quartz.stats.ExecStatData'
    ],

    /* Singleton stores that are automatically instantiated and registered with the Ext.StoreManager */
    stores : [
      'EmptyStore',

      'LocaleStore',
      'ConnectionsTreeStore',

      'enums.ExecNotificationRuleConditionTypeStore',
      'enums.ExecStatusStore',
      'enums.ExpressionLanguageStore',
      'enums.JsLoggingLevelStore',
      'enums.MessageChannelStore',
      'enums.NotificationSeverityStore',
      'enums.ProxyTypeStore',
      'enums.SnmpV3AuthenticationProtocolStore',
      'enums.SnmpV3PrivacyProtocolStore',
      'enums.SnmpV3SecurityLevelStore',
      'enums.WSAuthenticationTypeStore',
      'enums.WSVersionStore',

      'enums.JmxProtocolStore',

      'quartz.ExecNotificationRuleJSScriptingObjectsStore',
      'quartz.ExecNotificationRuleMessageMacrosStore',
      'quartz.stats.StatisticsTimePeriodLengthStore',
      'quartz.stats.JobStatisticsTypeStore',
      'quartz.stats.SchedulerStatisticsTypeStore',
      'quartz.stats.TriggerStatisticsTypeStore'
    ],

    /*
     * Executed by ExtJS from Ext.onReady
     */
    launch : function ()
    {
      /* Start of ExtJS overrides */

      Ext.override( Ext.data.proxy.Server, {
        /* Parameter names used in Ajax data store requests */
        startParam : 'startAt',
        groupParam : 'groupBy',
        sortParam : 'sort',
        filterParam : 'filter',

        noCache : true  // true is default (a _dc param s appended to all proxy request URLs)
      } );

      Ext.override( Ext.data.proxy.Ajax, {
        /* Default timeout in ms for all Ajax proxies */
        timeout : 1000 * QDesk.Config.get( 'ajaxTimeout' )
      } );

      Ext.override( Ext.data.Connection, {
        disableCaching : false, // false is default
        /* Default connect timeout in ms */
        timeout : 1000 * QDesk.Config.get( 'ajaxTimeout' )
      } );

      Ext.override( Ext.form.Basic, {
        /* Default timeout (in seconds) for all forms (used by the form load operation) */
        timeout : QDesk.Config.get( 'ajaxTimeout' )
      } );


      Ext.override( Ext.draw.engine.ImageExporter, {
        /*
         * Replaces the default ExtJS SVG export action URL "http://svg.sencha.io".
         */
        defaultUrl : QDesk.util.constant.ActionUrl.svgActionRasterize
      } );


      /*
       * Forms are validated after rendering so that invalid fields are immediately visually
       * distinguished.
       *
       * Form validation also occurs upon form field change.
       */
      Ext.override( Ext.form.Panel, {
        initComponent : function ()
        {
          var me = this;

          me.callParent( arguments );

          me.on( 'afterrender', function ()
          {
            me.isValid();
          } );

          Ext.each( Ext.ComponentQuery.query( 'field', me ), function ( field )
          {
            field.on( 'change', function ()
            {
              me.isValid();
            } );
          } );
        }
      } );


      /**
       * setCursorPosition and getCursorPosition for textfield, textarea etc.
       * http://www.sencha.com/forum/showthread.php?95486-Cursor-Position-in-TextField
       */
      Ext.override( Ext.form.field.Text, {

        setCaretPosition : function ( pos )
        {
          var el = this.inputEl.dom;
          if ( typeof(el.selectionStart) === "number" )
          {
            el.focus();
            el.setSelectionRange( pos, pos );
          }
          else if ( el.createTextRange )
          {
            var range = el.createTextRange();
            range.move( "character", pos );
            range.select();
          }
          else
          {
            //throw 'setCaretPosition() not supported';
          }
        },

        getCaretPosition : function ()
        {
          var el = this.inputEl.dom;
          if ( typeof(el.selectionStart) === "number" )
          {
            return el.selectionStart;
          }
          else if ( document.selection && el.createTextRange )
          {
            var range = document.selection.createRange();
            range.collapse( true );
            range.moveStart( "character", -el.value.length );
            return range.text.length;
          }
          else
          {
            //throw 'getCaretPosition() not supported';
          }
        }
      } );


      /**
       * JSON encoded Date instances are serialized using the ISO8601 format. Without this override the
       * format would be: yyyy-mm-ddThh:mm:ss (i.e. no time zone).
       */
      Ext.override( Ext.JSON, {
        encodeDate : function ( d )
        {
          return '"' + Ext.Date.format( d, QDesk.util.constant.DateFormat.timestampFormatIso8601 ) + '"';
        }
      } );


      /**
       * Localization of Ext.ux.form.ColorPickerField texts.
       */
      Ext.override( Ext.ux.form.ColorPickerField, {
        invalidValueError : QDesk.Msg.get( 'Ext.ux.form.ColorPickerField.invalidValueError' )
      } );


      /**
       * Localization of Ext.ux.TreeIconCombo texts.
       */
      Ext.override( Ext.ux.form.TreeIconCombo, {
        emptyValueText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.valueNotFoundText' ),
        valueNotFoundText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.valueNotFoundText' ),
        loadFailedText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.loadFailedText' ),
        loadingText : QDesk.Msg.get( 'Ext.ux.form.TreeIconCombo.loadingText' )
      } );


      /**
       * Default datetime format of the datetime value used by the DateTime component.
       */
      Ext.override( Ext.ux.form.field.DateTime, {
        dateTimeFormat : QDesk.util.constant.DateFormat.timestampFormatLong
      } );


      /* Start of Temporary fixes */

      /*
       * This fixes the problem with the time field ignoring the seconds and milliseconds by
       * setValue( Date ).
       *
       * Sencha does not seem to consider this a bug, but we do. This problem is reported at
       * http://www.sencha.com/forum/showthread.php?275972-TimeField-don-t-consider-the-seconds-and-milliseconds
       */

      Ext.override( Ext.form.field.Time, {
        getInitDate : function ( hours, minutes, seconds, milliseconds )
        {
          var parts = this.initDateParts;
          return new Date( parts[0], parts[1], parts[2], hours || 0, minutes || 0, seconds || 0, milliseconds || 0 );
        },

        setValue : function ( v )
        {
          this.getPicker();
          if ( Ext.isDate( v ) )
          {
            v = this.getInitDate( v.getHours(), v.getMinutes(), v.getSeconds(), v.getMilliseconds() );
          }
          return this.callSuper( [v] );
        }
      } );

      /* End of Temporary fixes */

      /* End of ExtJS overrides */

      Ext.create( 'QDesk.view.Viewport' );
    }
  } );
})();
