/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Triggers view in the scheduler view panel's Jobs tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.jobs.RefreshJobTriggersView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected job.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var jobsTab = QDesk.controller.QuartzControllerUtils.getJobsTab( me.connectionId );
    var jobsGrid = jobsTab.down( '#jobsGrid' );

    var selectedJobDetails = jobsGrid.getSelectionModel().getSelection();

    var triggersGrid = jobsTab.down( '#jobTriggersGrid' );

    var filterByTriggerGroupCombo = triggersGrid.down( '#filterByTriggerGroupCombo' );

    // reloads and resets the trigger group filter combo and then reloads the job triggers grid
    var filterByTriggerGroupComboStore = filterByTriggerGroupCombo.getStore();

    var reloadStores = true;

    if ( me.populate )
    {
      if ( Ext.isEmpty( selectedJobDetails ) )
      {
        filterByTriggerGroupComboStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
        filterByTriggerGroupComboStore.getProxy().extraParams = {};
      }
      else
      {
        var jobDetail = selectedJobDetails[0];

        filterByTriggerGroupComboStore.getProxy().url =
            QDesk.util.constant.ActionUrl.quartz.triggerGroupActionListForJob;

        filterByTriggerGroupComboStore.getProxy().extraParams = {
          connectionId : me.connectionId,
          jobGroupName : jobDetail.get( 'group' ).name,
          jobName : jobDetail.get( 'name' )
        };
      }
    }
    else
    {
      if ( filterByTriggerGroupComboStore.getProxy().url == QDesk.util.constant.ActionUrl.emptyStoreAction )
        reloadStores = false;

      filterByTriggerGroupComboStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
      filterByTriggerGroupComboStore.getProxy().extraParams = {};
    }

    // optimization to avoid repeated reloads of empty stores
    if ( reloadStores )
    {
      // once the combo has been reloaded, reload the JobTriggersGrid combo
      filterByTriggerGroupComboStore.load( {
        scope : me,
        callback : function ( records, operation, success )
        {
          var triggersGridStore = triggersGrid.getStore();

          var jobDetail = selectedJobDetails[0];

          if ( jobDetail )
          {
            // if job is selected
            triggersGridStore.getProxy().url = QDesk.util.constant.ActionUrl.quartz.triggerActionListForJob;
            triggersGridStore.getProxy().extraParams = {
              connectionId : me.connectionId,
              jobGroupName : jobDetail.get( 'group' ).name,
              jobName : jobDetail.get( 'name' )
            };
          }
          else
          {
            // if no job is selected
            triggersGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
            triggersGridStore.getProxy().extraParams = {};
          }

          // reload the jobs grid
          if ( filterByTriggerGroupCombo.getValue() == null )
          {
            // no need to reset the combo, just reload the grid
            triggersGridStore.loadPage( 1 );
          }
          else
          {
            // triggers the change event which reloads the job grid
            filterByTriggerGroupCombo.reset( null );
          }
        }
      } );
    }
  }
} );
