/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * Refreshes the Notifications view in the scheduler view panel's Triggers tab.
 */
Ext.define( 'QDesk.controller.command.scheduler.quartz.triggers.RefreshTriggerExecNotificationsView', {
  extend : 'QDesk.controller.command.BaseCommand',

  requires : [
    'QDesk.Msg',
    'QDesk.controller.QuartzControllerUtils',
    'QDesk.controller.command.connection.ConnectionUtils',
    'QDesk.util.constant.ActionUrl',
    'QDesk.util.license.LicensedProductFeatures'
  ],

  /*
   * Required connection ID config parameter.
   */
  connectionId : undefined,

  /**
   * A required flag indicating whether the view should be populated with data to reflect the currently selected trigger.
   */
  populate : undefined,


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.connectionId )
      Ext.Error.raise( 'Required config parameter missing: connectionId' );

    if ( config.populate == null )
      Ext.Error.raise( 'Required config parameter missing: populate' );

    me.callParent( [config] );
  },


  execute : function ()
  {
    var me = this;

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    if ( licensedFeatures.TriggerExecutionNotifications &&
        QDesk.controller.command.connection.ConnectionUtils.isJvmAgentAvailableAndCompatible( me.connectionId ) )
    {
      var triggersTab = QDesk.controller.QuartzControllerUtils.getTriggersTab( me.connectionId );
      var triggersGrid = triggersTab.down( '#triggersGrid' );

      var selectedTriggers = triggersGrid.getSelectionModel().getSelection();

      var notificationRulesGrid = triggersTab.down( '#execNotificationRulesGrid' );
      var notificationRulesGridStore = notificationRulesGrid.getStore();

      var reloadStores = true;

      if ( me.populate )
      {
        if ( Ext.isEmpty( selectedTriggers ) )
        {
          notificationRulesGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
          notificationRulesGridStore.getProxy().extraParams = {};
        }
        else
        {
          var triggerWithJobDetail = selectedTriggers[0];
          var trigger = triggerWithJobDetail.get( 'trigger' );

          notificationRulesGridStore.getProxy().url =
              QDesk.util.constant.ActionUrl.quartz.execNotificationRuleActionListForTrigger;

          notificationRulesGridStore.getProxy().extraParams = {
            connectionId : me.connectionId,
            triggerGroupName : trigger.group.name,
            triggerName : trigger.name
          };
        }
      }
      else
      {
        if ( notificationRulesGridStore.getProxy().url == QDesk.util.constant.ActionUrl.emptyStoreAction )
          reloadStores = false;

        notificationRulesGridStore.getProxy().url = QDesk.util.constant.ActionUrl.emptyStoreAction;
        notificationRulesGridStore.getProxy().extraParams = {};
      }

      // optimization to avoid repeated reloads of empty stores
      if ( reloadStores )
        notificationRulesGridStore.loadPage( 1 );
    }
  }
} );
