/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

/**
 * The Quartz scheduler trigger data model based on the
 * {@link com.quartzdesk.domain.model.scheduler.quartz.Trigger} class.
 */
Ext.define( 'QDesk.model.quartz.Trigger', {
  extend : 'Ext.data.Model',

  requires : [
    'QDesk.model.TimeZone',
    'QDesk.model.quartz.Calendar',
    'QDesk.model.quartz.JobGroup',
    'QDesk.model.quartz.TriggerGroup',
    'QDesk.util.constant.DateFormat'
  ],

  /*    Trigger Type               1.8.5  1.8.6  2.0.0  2.0.1  2.0.2  2.1.0  2.1.1  2.1.2  2.1.3  2.1.4  2.1.5  2.1.6  2.1.7  2.2.0  2.2.1
   *    CronTrigger                  y      y      y      y      y      y      y      y      y      y      y      y      y      y      y
   *    SimpleTrigger                y      y      y      y      y      y      y      y      y      y      y      y      y      y      y
   *    CalendarIntervalTrigger      n      n      y      y      y      y      y      y      y      y      y      y      y      y      y
   *    DailyIntervalTimeTrigger     n      n      n      n      n      y      y      y      y      y      y      y      y      y      y
   *    DateIntervalTrigger          y      y      n      n      n      n      n      n      n      n      n      n      n      n      n
   *    NthIncludedDayTrigger        y      y      n      n      n      n      n      n      n      n      n      n      n      n      n
   */

  fields : [
    /* QDesk.enums.scheduler.quartz.TriggerType */
    { name : 'type', type : 'string', useNull : true },

    /*
     * Start of common trigger attributes.
     */
    { name : 'name', type : 'string', useNull : true },
    { name : 'group', type : 'QDesk.model.quartz.TriggerGroup' },

    /* com.quartzdesk.domain.model.scheduler.quartz.TriggerState */
    { name : 'state', type : 'string', useNull : true },

    { name : 'jobName', type : 'string', useNull : true },
    { name : 'jobGroup', type : 'QDesk.model.quartz.JobGroup' },

    { name : 'calendar', type : 'QDesk.model.quartz.Calendar' },

    { name : 'description', type : 'string', useNull : true },

    { name : 'priority', type : 'int', useNull : true },

    { name : 'misfireInstruction', type : 'int', useNull : true },

    /* All dates are expected in ISO 8601 format (dateFormat: QDesk.util.constant.DateFormat.timestampFormatIso8601) */
    { name : 'startTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601 },
    { name : 'endTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601},

    { name : 'previousFireTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601 },
    { name : 'nextFireTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601 },
    { name : 'finalFireTime', type : 'date', dateFormat : QDesk.util.constant.DateFormat.timestampFormatIso8601 },

    { name : 'jobDataMap', type : 'auto' },
    { name : 'effectiveJobDataMap', type : 'auto' },

    /*
     * Quartz 1.x.
     */
    { name : 'volatility', type : 'boolean' },

    /* End of common trigger attributes */

    /* Start of trigger-type specific attributes */

    { name : 'cronExpression', type : 'string', useNull : true }, // CronTrigger

    { name : 'timeZone', type : 'QDesk.model.TimeZone' },          // CronTrigger, CalendarIntervalTrigger (>=2.1.3), NthIncludedDayTrigger

    { name : 'repeatCount', type : 'int', useNull : true },       // SimpleTrigger, DailyTimeIntervalTrigger

    { name : 'repeatInterval', type : 'int', useNull : true },    // SimpleTrigger, CalendarIntervalTrigger, DailyTimeIntervalTrigger, DateIntervalTrigger
    /* com.quartzdesk.domain.model.scheduler.quartz.IntervalUnit */
    { name : 'repeatIntervalUnit', type : 'string', useNull : true }, // CalendarIntervalTrigger, DailyTimeIntervalTrigger, DateIntervalTrigger

    { name : 'preserveHourOfDayAcrossDaylightSavings', type : 'boolean', useNull : true },  // CalendarIntervalTrigger (>=2.1.3)
    { name : 'skipDayIfHourDoesNotExist', type : 'boolean', useNull : true },               // CalendarIntervalTrigger (>=2.1.3)

//    { name : 'startTimeOfDay', type : 'date', dateFormat : QDesk.util.constant.DateFormat.dayTimeFormat },   // DailyTimeIntervalTrigger
//    { name : 'endTimeOfDay', type : 'date', dateFormat : QDesk.util.constant.DateFormat.dayTimeFormat },     // DailyTimeIntervalTrigger

    { name : 'startTimeOfDay', type : 'string', useNull : true },   // DailyTimeIntervalTrigger
    { name : 'endTimeOfDay', type : 'string', useNull : true },     // DailyTimeIntervalTrigger

    /* List of QDesk.enums.Weekday */
    { name : 'daysOfWeek', type : 'auto' },   // DailyTimeIntervalTrigger

    { name : 'nthDay', type : 'int', useNull : true },                // NthIncludedDayTrigger
    /* com.quartzdesk.domain.model.scheduler.quartz.IntervalType */
    { name : 'intervalType', type : 'string', useNull : true },       // NthIncludedDayTrigger

//    { name : 'fireAt', type : 'date', dateFormat : QDesk.util.constant.DateFormat.dayTimeFormat },  // NthIncludedDayTrigger
    { name : 'fireAt', type : 'string', useNull : true },  // NthIncludedDayTrigger

    { name : 'nextFireCutoffInterval', type : 'int', useNull : true }, // NthIncludedDayTrigger

    /* End of trigger-type specific attributes */

    /*
     * QuartzDesk trigger monitoring URL.
     * Read-only property.
     */
    { name : 'monitoringUrl', type : 'string' }
  ]} );
