/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */


Ext.define( 'QDesk.util.quartz.QuartzEnumUtils', {
  requires : [
    'QDesk.Log',
    'QDesk.Msg'
  ],

  singleton : true,

  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.TriggerType value.
   * @param triggerType a QDesk.enums.scheduler.quartz.TriggerType value.
   * @return the localized repeat interval unit name.
   */
  getTriggerTypeName : function ( triggerType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.TriggerType.' + triggerType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.RepeatIntervalUnit value.
   * @param repeatIntervalUnit a QDesk.enums.scheduler.quartz.RepeatIntervalUnit value.
   * @return the localized repeat interval unit name.
   */
  getRepeatIntervalUnitName : function ( repeatIntervalUnit )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.RepeatIntervalUnit.' + repeatIntervalUnit );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.IntervalType value.
   * @param intervalType a QDesk.enums.scheduler.quartz.IntervalType value.
   * @return the localized interval type name.
   */
  getIntervalTypeName : function ( intervalType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.IntervalType.' + intervalType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.TriggerStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.TriggerStatisticsType value.
   * @return the localized JavaScript trigger statistics type name.
   */
  getTriggerStatisticsTypeName : function ( statisticsType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.TriggerStatisticsType.' + statisticsType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.JobStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.JobStatisticsType value.
   * @return the localized JavaScript job statistics type name.
   */
  getJobStatisticsTypeName : function ( statisticsType )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.JobStatisticsType.' + statisticsType );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.SchedulerStatisticsType value.
   * @param statisticsType a QDesk.enums.scheduler.quartz.SchedulerStatisticsType value.
   * @return the localized JavaScript scheduler statistics type name.
   */
  getSchedulerStatisticsTypeName : function ( statisticsType )
  {
    var args = Array.prototype.slice.call( arguments ).slice( 1 );
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.SchedulerStatisticsType.' + statisticsType, args );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getCalendarIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.CalendarIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getCronTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.CronTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getDailyTimeIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.DailyTimeIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getDateIntervalTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.DateIntervalTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getNthIncludedDayTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.NthIncludedDayTriggerMisfireInstruction.' +
        misfireInstruction );
  },


  /**
   * Returns a localized name for the specified QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction
   * value.
   * @param misfireInstruction a QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction value.
   * @return the localized JavaScript misfire instruction name.
   */
  getSimpleTriggerMisfireInstructionName : function ( misfireInstruction )
  {
    return QDesk.Msg.get( 'QDesk.enums.scheduler.quartz.SimpleTriggerMisfireInstruction.' +
        misfireInstruction );
  }
} );
