/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.Config',
    'QDesk.Msg',
    'QDesk.store.quartz.JobDetailStore',
    'QDesk.store.quartz.JobGroupStore',

    'QDesk.util.license.LicensedProductFeatures',

    'QDesk.view.widget.ReadOnlyCheckColumn',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzJobsGrid',

  cls : 'quartzJobsGrid',

  viewConfig : {
    stripeRows : true
  },

  autoScroll : true,

  //border : false,

  tbar : [
    {
      xtype : 'button',
      itemId : 'refreshButton',
      scale : 'small',
      iconCls : 'refresh',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipRefreshButton' ),
      disabled : true
    },
    '-',
    {
      xtype : 'button',
      itemId : 'triggerJobButton',
      scale : 'small',
      iconCls : 'triggerJob',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipTriggerJobButton' ),
      disabled : true
    },
    '-',
    {
      xtype : 'button',
      itemId : 'addJobButton',
      scale : 'small',
      iconCls : 'addJob',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipAddJobButton' ),
      disabled : true
    },
    {
      xtype : 'button',
      itemId : 'deleteJobButton',
      scale : 'small',
      iconCls : 'deleteJob',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipDeleteJobButton' ),
      disabled : true
    },
    {
      xtype : 'button',
      itemId : 'editJobButton',
      scale : 'small',
      iconCls : 'editJob',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipEditJobButton' ),
      disabled : true
    },
    {
      xtype : 'button',
      itemId : 'cloneJobButton',
      scale : 'small',
      iconCls : 'cloneJob',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipCloneJobButton' ),
      disabled : true
    },
    '-',
    {
      xtype : 'xcombo',
      itemId : 'filterByJobGroupCombo',
      //fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.fieldLabelFilterByJobGroupCombo' ),
      //labelWidth : 'auto',
      //labelAlign : 'right',
      displayField : 'name',
      valueField : 'name',
      editable : false,
      emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.emptyTextFilterByJobGroupCombo' ),
      //store: store,   // store set in initComponent
      disabled : true
    },
    {
      xtype : 'button',
      itemId : 'resumeJobGroupButton',
      scale : 'small',
      iconCls : 'resumeJobGroup',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipResumeJobGroupButton' ),
      disabled : true
    },
    {
      xtype : 'button',
      itemId : 'pauseJobGroupButton',
      scale : 'small',
      iconCls : 'pauseJobGroup',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipPauseJobGroupButton' ),
      disabled : true
    },
    {
      xtype : 'tbfill'
    },
    {
      xtype : 'button',
      itemId : 'saveGridConfigButton',
      scale : 'small',
      iconCls : 'saveGridConfig',
      tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipSaveGridConfigButton' ),
      disabled : false
    }
  ],


  columns : [
    {
      itemId : 'nameColumn',
      text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textNameColumn' ),
      dataIndex : 'name',
      sortable : true,
      groupable : false,
      flex : 0.5
    },
    {
      xtype : 'templatecolumn',
      itemId : 'groupNameColumn',
      text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textGroupNameColumn' ),
      dataIndex : 'group.name',
      tpl : '{group.name}',
      sortable : true,
      groupable : true,
      flex : 0.5
    },
    {
      itemId : 'jobClassNameColumn',
      text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textJobClassColumn' ),
      dataIndex : 'jobClass',
      sortable : true,
      groupable : true,
      flex : 1
    },
    {
      xtype : 'actioncolumn',
      itemId : 'jobDataMapColumn',
      align : 'center',
      menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.history.ExecHistoryGrid.textLogColumn' ),
      header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textJobDataMapColumn' ),
      sortable : false,
      groupable : false,
      width : 100,
      items : [
        {
          icon : 'images/icon/job-data-map-16x16.png',
          iconCls : 'clickable',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipJobDataMapColumn' ),
          handler : function ( grid, rowIndex, colIndex )
          {
            var jobDetailRecord = grid.getStore().getAt( rowIndex );

            var jobDataMap = jobDetailRecord.get( 'jobDataMap' );
            this.fireEvent( 'requestViewJobDataMap', jobDataMap );
          }
        }
      ]
    },
    {
      xtype : 'ReadOnlyCheckColumn',
      itemId : 'durabilityColumn',
      text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textDurabilityColumn' ),
      align : 'center',
      dataIndex : 'durability',
      sortable : false,
      groupable : false,
      width : 100
    },
    {
      xtype : 'ReadOnlyCheckColumn',
      itemId : 'shouldRecoverColumn',
      text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textShouldRecoverColumn' ),
      align : 'center',
      dataIndex : 'shouldRecover',
      sortable : false,
      groupable : false,
      width : 100
    },
    {
      xtype : 'ReadOnlyCheckColumn',
      itemId : 'volatilityColumn',
      text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.textVolatilityColumn' ),
      align : 'center',
      dataIndex : 'volatility',
      sortable : false,
      groupable : false,
      width : 100
    }
  ],


  plugins : [
    {
      ptype : 'rowexpander',
      rowBodyTpl : [
        '<p>' +
            '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
            '<tpl if="description">{[Ext.htmlEncode(values.description)]}</tpl>' +
            '<tpl if="!description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
            '</p>' +
            '<p style="white-space: nowrap">' +
            '<b>{[QDesk.Msg.get( \"QDesk.monitoringUrl\" )]}:</b> ' +
            '{[this.getMonitoringLink( values.monitoringUrl )]}' +
            '</p>',
        {
          getMonitoringLink : function ( monitoringUrl )
          {
            var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
            if ( licensedFeatures.JobMonitoring )
            {
              var tooltip = Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipMonitoringUrl' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
            }
            else
            {
              var tooltip = Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.jobs.JobsGrid.tooltipMonitoringUrlNotLicensed' ) );
              return '<a href="' + monitoringUrl +
                  '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                  tooltip + '">' + monitoringUrl + '</a>';
            }
          }
        }
      ]
    }
  ],


  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var jobGroupStore = Ext.create( 'QDesk.store.quartz.JobGroupStore' );

    jobGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByJobGroupCombo' ).store = jobGroupStore;
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    var jobDetailStore = Ext.create( 'QDesk.store.quartz.JobDetailStore' );

    jobDetailStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    Ext.apply( config, {
      store : jobDetailStore,

      qdesk : {
        gridId : 'quartz.jobsGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [config] );
  }
} );
