/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm', {
  extend : 'QDesk.view.base.BaseFormPanel',

  requires : [
    'QDesk.Msg',

    'QDesk.store.quartz.CalendarStore',

    'QDesk.util.EnumUtils',
    'QDesk.util.FormUtils',
    'QDesk.util.FormatUtils',
    'QDesk.util.UxUtils',
    'QDesk.util.constant.DateFormat',

    'QDesk.util.quartz.QuartzUtils',
    'QDesk.util.quartz.QuartzEnumUtils',

    'QDesk.view.base.BaseFormPanel',

    'QDesk.view.main.scheduler.quartz.JobDataMapGrid',
    'QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode',

    'Ext.ux.form.field.DateTime',
    'Ext.ux.form.ClearableCombo'
  ],

  bodyPadding : '5 10 10 10',

  fieldDefaults : {
    labelWidth : 100
  },

  qdesk : {
    /*
     * Mode of the form (see EditTriggerFormMode).
     */
    mode : undefined,

    /**
     * Trigger type (see QDesk.enums.scheduler.quartz.TriggerType).
     */
    triggerType : undefined,

    /*
     * Trigger job group name.
     */
    triggerJobGroupName : undefined,

    /*
     * Trigger job name.
     */
    triggerJobName : undefined,

    /*
     * JVM Agent info object (see QDesk.controller.command.connection.ConnectionUtils.getJvmAgentInfo).
     */
    jvmAgentInfo : undefined,

    /*
     * Scheduler info object (see QDesk.controller.command.connection.ConnectionUtils.getSchedulerInfo).
     */
    schedulerInfo : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.triggerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerType' );

    if ( !config.qdesk.triggerJobGroupName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobGroupName' );

    if ( !config.qdesk.triggerJobName )
      Ext.Error.raise( 'Required config parameter missing: qdesk.triggerJobName' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    var propertiesItems = me._getTriggerSpecificFormItems( config.qdesk );

    var calendarStore = Ext.create( 'QDesk.store.quartz.CalendarStore', {
      autoLoad : false
    } );

    calendarStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    Ext.apply( config, {
      items : [
        /* Start of hidden fields */
        {
          xtype : 'hiddenfield',
          itemId : 'type',
          name : 'type',
          value : config.qdesk.triggerType
        },
        /* End of hidden fields */

        {
          xtype : 'displayfield',
          cls : 'formHeadingText',
          value : me._getTriggerSpecificFormHeadingText( config.qdesk.triggerType )
        },

        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldsetLabelJob' ),
          padding : '0 10 0 10',
          defaults : {
            labelWidth : 50,
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'jobName',
              name : 'jobName',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelJobName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtJobName' ),
              allowBlank : false,
              readOnly : true,
              cls : 'readOnly',
              value : config.qdesk.triggerJobName
            },
            {
              xtype : 'textfield',
              itemId : 'jobGroupName',
              name : 'jobGroupName',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelJobGroup' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtJobGroup' ),
              allowBlank : false,
              readOnly : true,
              cls : 'readOnly',
              value : config.qdesk.triggerJobGroupName
            }
          ]
        },  // end of Job fieldset


        {
          xtype : 'fieldset',
          title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldsetLabelTrigger' ),
          padding : '0 10 0 10',
          defaults : {
            labelWidth : 100,
            anchor : '100%'
          },
          items : [
            {
              xtype : 'textfield',
              itemId : 'name',
              name : 'name',
              labelWidth : 50,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtName' ),
              allowBlank : false,
              readOnly : config.qdesk.mode == QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.EDIT,
              cls : config.qdesk.mode == QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.EDIT ?
                  'readOnly' : ''
            },
            {
              xtype : 'textfield',
              itemId : 'group',
              name : 'group',
              labelWidth : 50,
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelGroup' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtGroup' ),
              allowBlank : false,
              readOnly : config.qdesk.mode == QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.EDIT,
              cls : config.qdesk.mode == QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.EDIT ?
                  'readOnly' : ''
            },

            {
              xtype : 'textareafield',
              itemId : 'description',
              name : 'description',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelDescription' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtDescription' ),
              labelAlign : 'top',
              allowBlank : true
            },

            /* Trigger start time */
            {
              xtype : 'xdatetime',
              itemId : 'startTime',
              name : 'startTime',
              allowBlank : false,
              value : new Date(),
              timeFormat : QDesk.util.constant.DateFormat.timeFormatLong,
              dateConfig : {
                fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelStartTime' ),
                emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtStartTimeDate' )
              },
              timeConfig : {
                emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtStartTimeTime' ),
                increment : 30 /* 30 minutes */
              }
            },

            /* Trigger end time */
            {
              xtype : 'xdatetime',
              itemId : 'endTime',
              name : 'endTime',
              allowBlank : true,
              timeFormat : QDesk.util.constant.DateFormat.timeFormatLong,
              dateConfig : {
                fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelEndTime' ),
                emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtEndTimeDate' )
              },
              timeConfig : {
                emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtEndTimeTime' ),
                increment : 30 /* 30 minutes */
              }
            },

            {
              xtype : 'numberfield',
              itemId : 'priority',
              name : 'priority',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelPriority' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtPriority' ),
              value : 5, /* default priority value */
              minValue : 1,
              maxValue : 65535,
              allowBlank : false
            },

            {
              xtype : 'combo',
              itemId : 'misfireInstruction',
              name : 'misfireInstruction',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelMisfireInstruction' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtMisfireInstruction' ),
              allowBlank : false,
              editable : false,
              forceSelection : true,
              queryMode : 'local',
              store : me._getTriggerSpecificMisfireInstructionStore( config.qdesk.schedulerInfo.version ),
              valueField : 'value',
              displayField : 'valueLabel'
            },

            {
              xtype : 'xcombo',
              itemId : 'calendarName',
              name : 'calendarName',
              fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldLabelCalendarName' ),
              emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.fieldEmptyTxtCalendarName' ),
              allowBlank : true,
              editable : false,
              forceSelection : false,
              queryMode : 'local',
              store : calendarStore,
              valueField : 'id',
              displayField : 'displayName'
            },

            {
              xtype : 'tabpanel',
              padding : '0 0 10 0',
              items : [
                {
                  xtype : 'panel',
                  padding : '10 10 5 10',
                  border : false,
                  title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.tabLabelProperties' ),
                  layout : 'anchor',
                  defaults : {
                    labelWidth : 120,
                    anchor : '100%'
                  },
                  items : propertiesItems
                },
                {
                  xtype : 'panel',
                  bodyPadding : 0,
                  border : false,
                  iconCls : 'quartzJobDataMapTabIcon',
                  title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.EditTriggerBaseForm.tabLabelJobDataMap' ),
                  layout : 'anchor',
                  defaults : {
                    anchor : '100%'
                  },
                  items : [
                    {
                      xtype : 'QuartzJobDataMapGrid',
                      border : false,
                      itemId : 'jobDataMapGrid',
                      height : 150,

                      qdesk : {
                        editable : true,
                        jobDataMap : []
                      }
                    }
                  ]
                }
              ]
            }
          ]
        }  // end of Trigger fieldset
      ],

      buttons : [
        {
          itemId : 'cancelButton',
          text : QDesk.Msg.get( 'QDesk.buttonLabelCancel' )
        },
        {
          itemId : 'actionButton',
          text : me._getActionButtonText( config.qdesk.mode ),
          formBind : true
        }
      ]
    } );

    me.callParent( [config] );

    me._bindEventHandlers();

    me._setDefaultFormValues();
  },


  /**
   * Returns an array of all form stores that must be loaded before this form can be opened.
   */
  getFormStores : function ()
  {
    var me = this;

    var stores = [me.down( '#calendarName' ).getStore()];

    // append trigger type specific stores
    return stores.concat( me._getTriggerSpecificStores() );
  },


  /**
   * Loads the specified trigger record into this form.
   * @param trigger the trigger record.
   */
  loadRecord : function ( trigger )
  {
    var me = this;
    me.callParent( arguments );

    me.getForm().setValues( {
      group : trigger.get( 'group' ).name,
      calendarName : trigger.get( 'calendar' ).id
    } );

    QDesk.util.quartz.QuartzUtils.populateJobDataMap( me.down( '#jobDataMapGrid' ), trigger.get( 'jobDataMap' ) );
  },


  /**
   * Returns the job data map as an array.
   * @returns {Array} the array of job data map entries.
   */
  getJobDataMap : function ()
  {
    var me = this;
    return QDesk.util.quartz.QuartzUtils.createJobDataMap( me.down( '#jobDataMapGrid' ) );
  },


  /**
   * Binds event handlers to all form widgets. Sub-classes must not override this method, they
   * should override the _bindTriggerSpecificEventHandlers method.
   */
  _bindEventHandlers : function ()
  {
    var me = this;
    // nothing to bind here for generic trigger form fields
    me._bindTriggerSpecificEventHandlers();
  },


  /**
   * Sets the default values of individual form fields. Sub-classes must not override this method, they
   * should override the _setTriggerSpecificFormDefaultValues method to set trigger type specific default
   * form values.
   */
  _setDefaultFormValues : function ()
  {
    var me = this;
    me._setTriggerSpecificFormDefaultValues();
  },


  /**
   * Returns the array of trigger type specific form items. Sub-classes must
   * override this method.
   * @param qdesk a QuartzDesk object wrapping various parameters passed to the form upon its creation.
   * @returns {Array} the array of trigger type specific form items.
   * @private
   */
  _getTriggerSpecificFormItems : function ( qdesk )
  {
    Ext.Error.raise( 'Sub-classes must override the _getTriggerSpecificFormItems method.' );
  },


  /**
   * Returns the trigger type specific misfire instruction store. Sub-classes must
   * override this method.
   * @param schedulerVersion the scheduler version.
   * @returns {Store} the misfire instruction store.
   * @private
   */
  _getTriggerSpecificMisfireInstructionStore : function ( schedulerVersion )
  {
    Ext.Error.raise( 'Sub-classes must override the _getTriggerSpecificMisfireInstructionStore method.' );
  },


  /**
   * Binds event handlers to trigger type specific form widgets. Sub-classes can override
   * this method.
   */
  _bindTriggerSpecificEventHandlers : function ()
  {
  },


  /**
   * Sets the default values of individual form fields. Sub-classes can override
   * this method.
   */
  _setTriggerSpecificFormDefaultValues : function ()
  {
  },


  /**
   * Invoked by the base form to obtain the array of all trigger type specific stores from
   * sub-classes. Sub-classes can override this method.
   */
  _getTriggerSpecificStores : function ()
  {
    return [];
  },


  _getTriggerSpecificFormHeadingText : function ( triggerType )
  {
    return QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( triggerType );
  },


  _getActionButtonText : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.ADD:
        return QDesk.Msg.get( 'QDesk.buttonLabelAdd' );

      case QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.EDIT:
        return QDesk.Msg.get( 'QDesk.buttonLabelUpdate' );

      case QDesk.view.main.scheduler.quartz.triggers.EditTriggerFormMode.CLONE:
        return QDesk.Msg.get( 'QDesk.buttonLabelClone' );

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
        return null;
    }
  }
} );
