/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid', {
  extend : 'QDesk.view.widget.ConfigurableGrid',

  requires : [
    'QDesk.Config',
    'QDesk.Msg',

    'QDesk.enums.scheduler.quartz.TriggerType',

    'QDesk.util.FormatUtils',

    'QDesk.store.quartz.TriggerGroupStore',
    'QDesk.store.quartz.TriggerWithJobDetailStore',

    'QDesk.view.widget.ReadOnlyCheckColumn',

    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.util.quartz.QuartzEnumUtils',
    'QDesk.util.quartz.QuartzTriggerUtils',

    'Ext.grid.plugin.RowExpander',
    'Ext.ux.form.ClearableCombo'
  ],

  alias : 'widget.QuartzTriggersGrid',

  cls : 'quartzTriggersGrid',

  autoScroll : true,

  //border : false,

  initComponent : function ()
  {
    var me = this;

    me.callParent();

    var triggerGroupStore = Ext.create( 'QDesk.store.quartz.TriggerGroupStore' );

    triggerGroupStore.getProxy().extraParams = {
      connectionId : me.qdesk.connectionId
    };

    me.down( '#filterByTriggerGroupCombo' ).store = triggerGroupStore;
  },


  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var schedulerVersion = config.qdesk.schedulerInfo.version;
    var jvmAgentAvailableAndCompatible = config.qdesk.jvmAgentInfo.compatible;

    Ext.apply( config, {
      tbar : [
        {
          xtype : 'button',
          itemId : 'refreshButton',
          scale : 'small',
          iconCls : 'refresh',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipRefreshButton' ),
          disabled : true
        },
        '-',
        {
          xtype : 'button',
          itemId : 'resumeTriggerButton',
          scale : 'small',
          iconCls : 'resumeTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipResumeTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerButton',
          scale : 'small',
          iconCls : 'pauseTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipPauseTriggerButton' ),
          disabled : true
        },
        '-',

        {
          xtype : 'button',
          itemId : 'addTriggerButton',
          scale : 'small',
          iconCls : 'addTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipAddTriggerButton' ),
          menu : {
            items : QDesk.util.quartz.QuartzTriggerUtils.createAddTriggerButtonMenuItems( schedulerVersion )
          },
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'deleteTriggerButton',
          scale : 'small',
          iconCls : 'deleteTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipDeleteTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'editTriggerButton',
          scale : 'small',
          iconCls : 'editTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipEditTriggerButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'cloneTriggerButton',
          scale : 'small',
          iconCls : 'cloneTrigger',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipCloneTriggerButton' ),
          disabled : true
        },
        '-',

        {
          xtype : 'xcombo',
          itemId : 'filterByTriggerGroupCombo',
          //fieldLabel : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.filterByJobGroupComboLabel' ),
          //labelWidth : 'auto',
          //labelAlign : 'right',
          displayField : 'name',
          valueField : 'name',
          editable : false,
          emptyText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.emptyTextFilterByTriggerGroupCombo' ),
          //store: store,   // store set in initComponent
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'resumeTriggerGroupButton',
          scale : 'small',
          iconCls : 'resumeTriggerGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipResumeTriggerGroupButton' ),
          disabled : true
        },
        {
          xtype : 'button',
          itemId : 'pauseTriggerGroupButton',
          scale : 'small',
          iconCls : 'pauseTriggerGroup',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipPauseTriggerGroupButton' ),
          disabled : true
        },
        {
          xtype : 'tbfill'
        },
        {
          xtype : 'button',
          itemId : 'saveGridConfigButton',
          scale : 'small',
          iconCls : 'saveGridConfig',
          tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipSaveGridConfigButton' ),
          disabled : false
        }
      ]
    } );

    Ext.apply( config, {
      columns : [
        {
          xtype : 'templatecolumn',
          itemId : 'triggerNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerNameColumn' ),
          dataIndex : 'trigger.name',
          tpl : '{trigger.name}',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerGroupNameColumn' ),
          dataIndex : 'trigger.group.name',
          tpl : '{trigger.group.name}',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerTypeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerTypeColumn' ),
          dataIndex : 'trigger.type',
          tpl : '{[Ext.isEmpty( values.trigger.type ) ? QDesk.Msg.get("QDesk.unknown") : QDesk.util.quartz.QuartzEnumUtils.getTriggerTypeName( values.trigger.type )]}',
          sortable : true,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerStateColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerStateColumn' ),
          dataIndex : 'trigger.state',
          tpl : '{trigger.state}',
          sortable : true,
          groupable : true,
          flex : 0.5
        },
        {
          xtype : 'actioncolumn',
          itemId : 'triggerJobDataMapColumn',
          align : 'center',
          menuText : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerJobDataMapColumn' ),
          header : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerJobDataMapColumn' ),
          sortable : false,
          groupable : false,
          width : 100,
          items : [
            {
              icon : 'images/icon/job-data-map-16x16.png',
              iconCls : 'clickable',
              tooltip : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipTriggerJobDataMapColumn' ),
              handler : function ( grid, rowIndex, colIndex )
              {
                var triggerWithJobDetailRecord = grid.getStore().getAt( rowIndex );
                var trigger = triggerWithJobDetailRecord.get( 'trigger' );

                var jobDataMap = trigger.jobDataMap;
                var effectiveJobDataMap = trigger.effectiveJobDataMap;
                this.fireEvent( 'requestViewJobDataMap', jobDataMap, effectiveJobDataMap );
              }
            }
          ]
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerStartTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerStartTimeColumn' ),
          dataIndex : 'trigger.startTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.startTime ) ]}',
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerEndTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerEndTimeColumn' ),
          dataIndex : 'trigger.endTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.endTime ) ]}',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerPreviousFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerPreviousFireTimeColumn' ),
          dataIndex : 'trigger.previousFireTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.previousFireTime ) ]}',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerNextFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerNextFireTimeColumn' ),
          dataIndex : 'trigger.nextFireTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.nextFireTime ) ]}',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerFinalFireTimeColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerFinalFireTimeColumn' ),
          dataIndex : 'trigger.finalFireTime',
          tpl : '{[ QDesk.util.FormatUtils.parseAndFormatTimestamp( values.trigger.finalFireTime ) ]}',
          sortable : true,
          groupable : true,
          flex : 1.1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerCalendarNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerCalendarNameColumn' ),
          dataIndex : 'trigger.calendar',
          tpl : '{trigger.calendar.displayName}',
          sortable : false,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerPriorityColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerPriorityColumn' ),
          dataIndex : 'trigger.priority',
          tpl : '{trigger.priority}',
          sortable : false,
          groupable : false,
          flex : 0.5
        },
        {
          xtype : 'templatecolumn',
          itemId : 'triggerMisfireInstructionColumn',
          format : '0',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textTriggerMisfireInstructionColumn' ),
          dataIndex : 'trigger.misfireInstruction',
          tpl : '{trigger.misfireInstruction}',
          sortable : false,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'jobNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobNameColumn' ),
          dataIndex : 'jobDetail.name',
          tpl : '{jobDetail.name}',
          sortable : true,
          groupable : false,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'jobGroupNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobGroupNameColumn' ),
          dataIndex : 'jobDetail.group.name',
          tpl : '{jobDetail.group.name}',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'templatecolumn',
          itemId : 'jobClassNameColumn',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobClassColumn' ),
          dataIndex : 'jobDetail.jobClass',
          tpl : '{jobDetail.jobClass}',
          sortable : true,
          groupable : true,
          flex : 1
        },
        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'jobDurabilityColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobDurabilityColumn' ),
          dataIndex : 'jobDetail.durability',
          //tpl : '{jobDetail.durability}',
          sortable : false,
          groupable : false,
          width : 100
        },
        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'jobShouldRecoverColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobShouldRecoverColumn' ),
          dataIndex : 'jobDetail.shouldRecover',
          //tpl : '{jobDetail.shouldRecover}',
          sortable : false,
          groupable : false,
          width : 100
        },
        {
          xtype : 'ReadOnlyCheckColumn',
          itemId : 'jobVolatilityColumn',
          align : 'center',
          text : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.textJobVolatilityColumn' ),
          dataIndex : 'jobDetail.volatility',
          //tpl : '{jobDetail.volatility}',
          //sortable : false,
          groupable : false,
          width : 100
        }
      ]
    } );

    Ext.apply( config, {
      plugins : [
        {
          ptype : 'rowexpander',
          rowBodyTpl : [
            '<p>' +
                '<b>{[QDesk.Msg.get( \"QDesk.description\" )]}:</b> ' +
                '<tpl if="trigger.description">{[Ext.htmlEncode(values.trigger.description)]}</tpl>' +
                '<tpl if="!trigger.description">{[Ext.htmlEncode(QDesk.Msg.get( \"QDesk.notAvailable\" ))]}</tpl>' +
                '</p>' +

                '{[QDesk.util.quartz.QuartzTriggerUtils.createTriggerPropertiesTable( values.trigger )]}' +

                '<p style="white-space: nowrap">' +
                '<b>{[QDesk.Msg.get( \"QDesk.monitoringUrl\" )]}:</b> ' +
                '{[this.getMonitoringLink( values.trigger.monitoringUrl )]}' +
                '</p>',
            {
              getMonitoringLink : function ( monitoringUrl )
              {
                var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;
                if ( licensedFeatures.TriggerMonitoring )
                {
                  var tooltip = Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipMonitoringUrl' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow" target="_blank" data-qtip="' + tooltip + '">' + monitoringUrl + '</a>';
                }
                else
                {
                  var tooltip = Ext.htmlEncode( QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid.tooltipMonitoringUrlNotLicensed' ) );
                  return '<a href="' + monitoringUrl +
                      '" class="newWindow notLicensed" onclick="javascript:return false" target="_blank" data-qtip="' +
                      tooltip + '">' + monitoringUrl + '</a>';
                }
              }
            }
          ]
        }
      ]
    } );

    var triggersStore = Ext.create( 'QDesk.store.quartz.TriggerWithJobDetailStore' );

    triggersStore.getProxy().extraParams = {
      connectionId : config.qdesk.connectionId
    };

    Ext.apply( config, {
      store : triggersStore,

      qdesk : {
        gridId : 'quartz.triggersGrid',
        connectionId : config.qdesk.connectionId
      }
    } );

    me.callParent( [config] );
  }
} );
