/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.main.scheduler.quartz.triggers.TriggersTab', {
  extend : 'QDesk.view.base.scheduler.BaseSchedulerSubTab',

  requires : [
    'QDesk.Msg',
    'QDesk.util.license.LicensedProduct',
    'QDesk.util.license.LicensedProductFeatures',
    'QDesk.view.main.scheduler.common.WarningMessagePanel',
    'QDesk.view.main.scheduler.quartz.triggers.TriggersGrid',
    'QDesk.view.main.scheduler.quartz.triggers.TriggersTabViews'
  ],

  alias : 'widget.QuartzTriggersTab',

  iconCls : 'quartzTriggersSubTabIcon',

  layout : {
    type : 'vbox',
    align : 'stretch'
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.connectionId )
      Ext.Error.raise( 'Required config parameter missing: qdesk.connectionId' );

    if ( !config.qdesk.schedulerInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerInfo' );

    if ( !config.qdesk.jvmAgentInfo )
      Ext.Error.raise( 'Required config parameter missing: qdesk.jvmAgentInfo' );

    var licensedFeatures = QDesk.util.license.LicensedProductFeatures.featureSets.Quartz;

    var items = [];
    if ( licensedFeatures.MaxJobsPerScheduler != QDesk.util.license.LicensedProductFeatures.UNLIMITED_INT
        || licensedFeatures.MaxTriggersPerJob != QDesk.util.license.LicensedProductFeatures.UNLIMITED_INT )
    {
      items.push( {
        xtype : 'WarningMessagePanel',
        qdesk : {
          msg : QDesk.Msg.get( 'QDesk.featureAccessRestricted.maxJobsPerSchedulerAndMaxTriggersPerJob',
              QDesk.util.license.LicensedProduct.productName, licensedFeatures.MaxJobsPerScheduler,
              licensedFeatures.MaxTriggersPerJob )
        }
      } );
    }

    items.push( {
      xtype : 'QuartzTriggersGrid',
      itemId : 'triggersGrid',
      flex : 1,
      qdesk : config.qdesk
    } );

    items.push( {
      xtype : 'QuartzTriggersTabViews',
      itemId : 'views',
      flex : 2,
      qdesk : config.qdesk
    } );

    Ext.apply( config, {
      title : QDesk.Msg.get( 'QDesk.view.main.scheduler.quartz.triggers.TriggersTab.title' ),
      items : items
    } );

    me.callParent( [config] );
  }
} );
