/*
 * Copyright (c) 2011-2014 QuartzDesk.com. All Rights Reserved.
 * QuartzDesk.com PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

Ext.define( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow', {
  extend : 'Ext.window.Window',

  requires : [
    'QDesk.view.navigation.connections.EditSchedulerConnectionForm',
    'QDesk.view.navigation.connections.EditSchedulerConnectionFormMode',
    'QDesk.Msg'
  ],

  alias : 'widget.EditSchedulerConnectionWindow',

  cls : 'editSchedulerConnectionWindow',
  iconCls : 'editSchedulerConnectionWindowIcon',

  modal : true,
  resizable : false,

  minWidth : 550,
  width : 550,

  qdesk : {
    /*
     * Mode of the form (see EditSchedulerConnectionFormMode).
     */
    mode : undefined,

    /*
     * Scheduler type (see QDesk.enums.scheduler.SchedulerType).
     */
    schedulerType : undefined
  },

  constructor : function ( config )
  {
    var me = this;

    config = config || {};

    if ( !config.qdesk )
      Ext.Error.raise( 'Required config parameter missing: qdesk' );

    if ( !config.qdesk.mode )
      Ext.Error.raise( 'Required config parameter missing: qdesk.mode' );

    if ( !config.qdesk.schedulerType )
      Ext.Error.raise( 'Required config parameter missing: qdesk.schedulerType' );

    Ext.apply( config, {
      title : me._getWindowTitle( config.qdesk.mode ),
      items : [
        {
          xtype : 'EditSchedulerConnectionForm',
          itemId : 'form',

          qdesk : {
            mode : config.qdesk.mode,
            schedulerType : config.qdesk.schedulerType
          }
        }
      ]
    } );

    me.callParent( [config] );

    me.down( '#cancelButton' ).on( 'click', function ()
    {
      me.close();
    } )
  },


  /**
   * Returns the title of this window.
   * @param mode a QDesk.view.navigation.connections.EditSchedulerConnectionFormMode value.
   */
  _getWindowTitle : function ( mode )
  {
    switch ( mode )
    {
      case QDesk.view.navigation.connections.EditSchedulerConnectionFormMode.ADD:
        title = QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow.windowTitleAdd' );
        break;

      case QDesk.view.navigation.connections.EditSchedulerConnectionFormMode.EDIT:
        title = QDesk.Msg.get( 'QDesk.view.navigation.connections.EditSchedulerConnectionWindow.windowTitleEdit' );
        break;

      default:
        Ext.Error.raise( "Unrecognized form mode: " + mode );
    }

    return title;
  }
} )
;
